// ShmMgr.cpp: implementation of the CShmMgr class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"


//
// PROJECT INCLUDE
//
#include "ShmMgrClient.h"
#include "ShmLoggerClient.h"
 

using namespace std;


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CShmLogClient::~CShmLogClient()
{
	//
	// All other members can be auto deleted
	//
}
  

 
CShmLogClient::CShmLogClient(	PVOID MemoryAddress, 
						ULONG MemorySize,
						std::string& KeyPrefix):
m_ShmBuffer((LOGBUFFER*)MemoryAddress),
m_MaxSize(MemorySize),
m_strKeyPrefix(KeyPrefix),
m_LogEntrySemPtr(new CMclSemaphore((KeyPrefix+SEMAPHOE_POSTFIX_PRODUCT).c_str())),
m_LogSpaceSemPtr(new CMclSemaphore((KeyPrefix+SEMAPHOE_POSTFIX_AVAIL).c_str()))
{
	m_bIsValid=	CMclIsValidHandle((*m_LogEntrySemPtr).GetHandle())&&
				CMclIsValidHandle((*m_LogSpaceSemPtr).GetHandle())&&
				CMclIsValidHandle(m_StopNotify.GetHandle())&&
				CMclIsValidHandle(m_StopAck.GetHandle()); 

	if (m_bIsValid)
	{
		if(InterlockedCompareExchange((LONG*)(&m_ShmBuffer->ulSig),
						0XBEEF, 0XBEEF) == 0XBEEF) 
		{
			m_bIsValid=true;
		}
		else
		{
			m_bIsValid=false;
		}
	}
		
}


 CShmLogClient* CShmLogClient::CreateClient(PVOID addr, ULONG size, std::string& strPrefix)
{
	CShmLogClient* p= new CShmLogClient(addr,size,strPrefix);

	if (p)
	{
		if (*p)
		{
			return p;
		}
		else
		{
			p->Delete();
		}
	}
	return NULL;
}

bool CShmLogClient::WriteString (std::vector<char>& data)
{
	if (! *this)
	{
		return false;
	}

	int totalSize=data.size();
	if (totalSize<EntryPayloadSize)
	{
		return WriteToBuffer(data, false, true);
	}


	int currentStartPoint=0;
	while (totalSize>EntryPayloadSize)
	{
		std::vector<char> writeData;
		writeData.resize(EntryPayloadSize);
		for(int i=0; i<EntryPayloadSize;i++)
		{
			writeData[i]=data[currentStartPoint+i];
		}
		currentStartPoint+=i;
		totalSize-=EntryPayloadSize;
		if (!WriteToBuffer(writeData,true,false))
		{
			return false;
		}
 	}

	std::vector<char> writeData;
	writeData.resize(totalSize);
		
	for(int i=0; i<totalSize;i++)
	{
		writeData[i]=data[currentStartPoint+i];
	}
	return WriteToBuffer(writeData,true,true);
}

bool CShmLogClient::WriteString (const char* data)
{
	if (! *this)
	{
		return false;
	}

	int totalSize=strlen(data);
	if (totalSize<EntryPayloadSize)
	{
		return WriteToBuffer(data, false, true);
	}


	int currentStartPoint=0;
	while (totalSize>EntryPayloadSize)
	{
		std::vector<char> writeData;
		writeData.resize(EntryPayloadSize);
		for(int i=0; i<EntryPayloadSize;i++)
		{
			writeData[i]=data[currentStartPoint+i];
		}
		currentStartPoint+=i;
		totalSize-=EntryPayloadSize;
		if (!WriteToBuffer(writeData,true,false))
		{
			return false;
		}
 	}

	std::vector<char> writeData;
	writeData.resize(totalSize);
		
	for(int i=0; i<totalSize;i++)
	{
		writeData[i]=data[currentStartPoint+i];
	}
	return WriteToBuffer(writeData,true,true);
}
 

bool CShmLogClient::WriteToBuffer(std::vector<char>& data,
							   bool bFragmented,
							   bool endFragment)
{
	if (! *this)
	{
		return false;
	}

	if (data.size()>EntryPayloadSize)
	{
		return false;
	}

	DWORD waitRet=m_LogSpaceSemPtr->Wait(1000);
	if (CMclWaitSucceededIndex(waitRet)!=WAIT_OBJECT_0)
	{
		//
		// If no space available in one second. The CLM service is considered deadth.
		// we won't log anything afterwards.
		//
		m_bIsValid=false;
		return false;
	}

	// Get a big enough slot in the ring buffer
	if(InterlockedIncrement(&m_ShmBuffer->count) > m_ShmBuffer->MaxCount)
	{
		//
		// The memory is full
		//
		InterlockedDecrement(&m_ShmBuffer->count);
		return false;
	}

	DWORD write_location = InterlockedIncrement(&m_ShmBuffer->write);
	write_location%=m_ShmBuffer->MaxCount;


	PLOG_ENTRY plogEntry=(PLOG_ENTRY) (m_ShmBuffer->buffer+LOGENTRY_STRING_SIZE*write_location);

	plogEntry->size=EntryHeaderSize+data.size();
	plogEntry->flag=0;
	if (bFragmented)
	{
		plogEntry->flag|=0x80;
		if (!endFragment)
		{
			plogEntry->flag|=0x40;
		}
		else
		{
			plogEntry->flag=0x80;

		}
	}
	plogEntry->Handle=GetCurrentProcessId();
	std::copy(data.begin(), data.end(), plogEntry->buffer);

	
	LONG preCount=0;
	m_LogEntrySemPtr->Release(1,&preCount);

	return true;
}

bool CShmLogClient::WriteToBuffer(const char* data,
							   bool bFragmented,
							   bool endFragment)
{
	if (! *this)
	{
		return false;
	}

	int DataSize=strlen(data);

	if (DataSize>EntryPayloadSize)
	{
		return false;
	}

	DWORD waitRet=m_LogSpaceSemPtr->Wait(1000);
	if (CMclWaitSucceededIndex(waitRet)!=WAIT_OBJECT_0)
	{
		//
		// If no space available in one second. The CLM service is considered deadth.
		// we won't log anything afterwards.
		//
		m_bIsValid=false;
		return false;
	}

	// Get a big enough slot in the ring buffer
	if(InterlockedIncrement(&m_ShmBuffer->count) > m_ShmBuffer->MaxCount)
	{
		//
		// The memory is full
		//
		InterlockedDecrement(&m_ShmBuffer->count);
		return false;
	}

	DWORD write_location = InterlockedIncrement(&m_ShmBuffer->write);
	write_location%=m_ShmBuffer->MaxCount;


	PLOG_ENTRY plogEntry=(PLOG_ENTRY) (m_ShmBuffer->buffer+LOGENTRY_STRING_SIZE*write_location);

	plogEntry->size=EntryHeaderSize+DataSize;
	plogEntry->flag=0;
	if (bFragmented)
	{
		plogEntry->flag|=0x80;
		if (!endFragment)
		{
			plogEntry->flag|=0x40;
		}
		else
		{
			plogEntry->flag=0x80;

		}
	}
	plogEntry->Handle=GetCurrentProcessId();
	std::copy(data, data+DataSize, plogEntry->buffer);

	
	LONG preCount=0;
	m_LogEntrySemPtr->Release(1,&preCount);

	return true;
}


 
