// DebugLogHandler.cpp: implementation of the CDebugLogHandler class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
 
#include "ShmMgr.h"
#include "ShmLoggerMgr.h"
#include "DebugLogHandler.h"
#include "DebugLoggerAppender.h"



//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDebugLogHandler::CDebugLogHandler()
{
	SHMBLOCK block=CShmMgr::ServerInstance().GetDebugLogBlock();
	m_pShmLogger=NULL;

	if(!block.m_address||!block.m_size)
    {
		return;
	}

	//
	// Read the registry keys and configure the debuglog appender
	//

	std::string Name=MAGIC_STRING_DEBUG;

	CDebugLoggerAppender* pAppender= new CDebugLoggerAppender;
	if (pAppender)
	{
		if (*pAppender)
		{
			m_pShmLogger=CShmLogMgr::CreateServer(block.m_address,
												  block.m_size,
												  Name, 
												  pAppender);
		}
		else
		{
			delete pAppender;
		}
	}
	CLMAssert(m_pShmLogger!=NULL, "Failed to create logger server");
}

CDebugLogHandler::~CDebugLogHandler()
{
	if (m_pShmLogger)
	{
		delete m_pShmLogger;
	}

	CShmMgr::CleanupServerInstance();
}


unsigned CDebugLogHandler::ThreadHandlerProc(void)
{
	if (*this)
	{
		m_pShmLogger->ProcessLogs();
	}
	return 0;
}

void CDebugLogHandler::Start()
{
	CMclThread(this);
}

void CDebugLogHandler::Stop()
{
	if (*this)
	{
		m_pShmLogger->StopLogger();
	}
}


CDebugLogHandler& CDebugLogHandler::Instance()
{
	static CDebugLogHandler obj;
	return obj;

}

